/*
**		Semi Circle Demo Program For svgalib @ listbot
**		~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
/*
**=============================================================================
*/
/*
**		Includes
**		~~~~~~~~
*/

#include <stdio.h>
#include <stdlib.h>
#include <vga.h>
#include <vgagl.h>
#include <vgakeyboard.h>
#include <math.h>
#include <time.h>

/*
**=============================================================================
*/
/*
**		Defines
**		~~~~~~~
*/

#define sW 640
#define sH 480

#define MAX_X sW-1
#define MAX_Y sH-1

#define MAX_RADIUS 240

#define MODE G640x480x256

#define FIXED_B 16
#define FIXED_M (1<<FIXED_B)

#define RAD_DEG (180/M_PI)
#define DEG_RAD (M_PI/180)

/*
**=============================================================================
*/
/*
**		Global Variables
**		~~~~~~~~~~~~~~~~
*/

int y_min;
int y_max;
int x_min[sW];
int x_max[sW];

int starttime,endtime;
int amountdrawn = 0;

/*
**=============================================================================
*/
/*
**		Forward Declarations
**		~~~~~~~~~~~~~~~~~~~~
*/

void demo_semicircles();
void put_renderline( int x1, int y1, int x2, int y2);
void put_renderpixel( int x, int y );
void renderit(int colour);
void draw_semi_circle(int x,int y,int r, int st_a, int en_a, int colour);
void put_render_semi(int x,int y,int r, int st_a, int en_a);

/*
**=============================================================================
*/
/*
**		Procedures
**		~~~~~~~~~~
*/
/*
**=============================================================================
*/
int main () {
  int yc;

  vga_init();
  if ( keyboard_init() ) {
    fprintf( stderr , "Could not initialize keyboard.\n" );
    exit(-1);
  }
  vga_setmode(MODE);
  gl_setcontextvga(MODE);

/*	Clear The Draw Buffer
*/
  for (yc=0;yc<=MAX_Y;yc++) {
    x_max[yc]=-(x_min[yc]=sW<<1);
  }
  y_max=-(y_min=sH<<1);

/*	Lets go...
*/
  demo_semicircles();

  vga_setmode(TEXT); 
  
  if (endtime-starttime) {
    fprintf(stdout,
      "\nSemi-circles drawn : %d\nTime in seconds passed : %d\nApprox %d Semi-circles per second\n\n",
      amountdrawn,
      endtime-starttime,
      amountdrawn/(endtime-starttime));
  }      
  exit (0);
}

/*
**=============================================================================
*/

void demo_semicircles() {
  int angs,ange,temp;

  (time_t)starttime=time(NULL);
  ange=angs=0;
  while ( ! keyboard_keypressed(SCANCODE_ESCAPE) ) {
    draw_semi_circle(sW>>1,sH>>1,(sH>>1)+(int)(((sH>>1))*sin(ange*DEG_RAD)),angs,angs+45,1);
    draw_semi_circle(sW>>1,sH>>1,(sH>>1)+(int)(((sH>>1))*sin(ange*DEG_RAD)),angs+45,angs+360,0);
    angs++;
    ange+=3;
    angs%=360;
    ange%=360;
    keyboard_update();
  }
  while ( keyboard_keypressed(SCANCODE_ESCAPE) ) {keyboard_update();}
  while ( ! keyboard_keypressed(SCANCODE_ESCAPE) ) {

      angs=rand()%360; 
      ange=rand()%360;
      if (angs>ange) { temp=angs;angs=ange;ange=temp; }

      draw_semi_circle( (rand()%(sW<<1))-(sW>>1),
                        (rand()%(sH<<1))-(sH>>1),
                        (rand()%MAX_RADIUS),
                        angs,ange,rand()%256);
		       
     amountdrawn++;
     keyboard_update();
      
  }
  (time_t)endtime=time(NULL);
}

/*
**=============================================================================
*/

void put_renderline( int x1, int y1, int x2, int y2) {
  int temp;
  int xd,xp,yc,ax;
  
  if(y2<y1) {
    temp=y1;y1=y2;y2=temp;
    temp=x1;x1=x2;x2=temp;
  }
  
  if (y1>MAX_Y) return;
  if (y2<0) return;
  
  if (y1==y2) { xd=0; }
  else { xd=(((x2-x1)*FIXED_M)/(y2-y1)); }
  
  xp=x1*FIXED_M;
  
  if (y2>MAX_Y) y2=MAX_Y;
  if (y1<0) {
    xp+=(-y1)*xd;
    y1=0;
  }
  
  if (y1<y_min) y_min=y1;
  if (y2>y_max) y_max=y2;

  for(yc=y1;yc<=y2;yc++) {

    ax=xp/FIXED_M;

    if (ax<x_min[yc]) x_min[yc]=ax;
    if (ax>x_max[yc]) x_max[yc]=ax;
    xp+=xd;    
  }
}

/*
**=============================================================================
*/

void put_renderpixel( int x, int y ) {
  if ((y<0)||(y>MAX_Y)) return;
  if (y<y_min) y_min=y;
  if (y>y_max) y_max=y;
  if (x<x_min[y]) x_min[y]=x;
  if (x>x_max[y]) x_max[y]=x;
}

/*
**=============================================================================
*/

void renderit(int colour) {
  int yc;

  vga_setcolor(colour);
  
  for (yc=y_min;yc<=y_max;yc++) {
    if ((x_min[yc]<MAX_X)&&(x_max[yc]>0)) {
      if (x_min[yc]<0) x_min[yc]=0;
      if (x_max[yc]>MAX_X) x_max[yc]=MAX_X;
      gl_hline(x_min[yc],yc,x_max[yc],colour); 
    }
  }
  for (yc=0;yc<=MAX_Y;yc++) {
    x_max[yc]=-(x_min[yc]=sW<<1);
  }
  y_max=-(y_min=sH<<1);
}

/*
**=============================================================================
*/

void draw_semi_circle(int x,int y,int r, int st_a, int en_a, int colour) {

  if (r<1) return;
  
  st_a%=360;
  en_a%=360;
  if (en_a<st_a) en_a+=360;

  if ((en_a-st_a)>180) {
    put_render_semi( x, y, r, st_a, st_a+180);
    renderit(colour);
    st_a+=180;
  }
  put_render_semi( x, y, r, st_a, en_a);
  renderit(colour);

}

void put_render_semi(int x,int y,int r, int st_a, int en_a) {
  int yc;
  int cx,cu;
  double ca;
  int dx,dy;
  
  for (yc=r;yc>0;yc--) {

    cu=((int)((ca=asin((double)yc/(double)r))*RAD_DEG));
    cx=(int)(cos(ca)*(double)r);

    if (((st_a<=cu)&&(cu<=en_a))||
        (((cu)<st_a)&&(st_a<=(cu+360))&&((cu+360)<=en_a))) {
      put_renderpixel(x+cx,y-yc);
    }
    if ( ((st_a<=((180-cu)))&&(((180-cu))<=en_a))||
         (((180-cu)<st_a)&&(st_a<=((540-cu)))&&(((540-cu))<=en_a))) {
      put_renderpixel(x-cx,y-yc);
    }
    if (((st_a<=((180+cu)))&&(((180+cu))<=en_a))||
        (((180+cu)<st_a)&&(st_a<=((540+cu)))&&(((540+cu))<=en_a))) {
      put_renderpixel(x-cx,y+yc);
    }
    if (((st_a<=((360-cu)))&&(((360-cu))<=en_a))||
        (((360-cu)<st_a)&&(st_a<=((720-cu)))&&(((720-cu))<=en_a))) {
      put_renderpixel(x+cx,y+yc);
    }
  }

  if (((st_a<=0)&&(0<=en_a))||
      ((st_a<=360)&&(360<=en_a))) put_renderpixel(x+r,y);
  if (((st_a<=180)&&(180<=en_a))||
      ((180<st_a)&&(st_a<=540)&&(540<=en_a))) put_renderpixel(x-r,y);
  
  
  dx=(int)((double)x+cos((double)st_a*DEG_RAD)*(double)r);
  dy=(int)((double)y-sin((double)st_a*DEG_RAD)*(double)r);
  put_renderline( x,y,dx,dy);

  dx=(int)((double)x+cos((double)en_a*DEG_RAD)*(double)r);
  dy=(int)((double)y-sin((double)en_a*DEG_RAD)*(double)r);
  put_renderline( x,y,dx,dy);

}

/*
**=============================================================================
*/
